@@ -23,7 +23,8 @@ from typing import List, Any, Optional, Union, Dict, AsyncIterator, AsyncGenerat
 import pytest
 from _pytest.reports import TestReport
 
-from pytest_mproc import resource_utilization, find_free_port, user_output, DEFAULT_PRIORITY, get_auth_key, FatalError
+from pytest_mproc import resource_utilization, find_free_port, user_output, DEFAULT_PRIORITY, get_auth_key, FatalError, \
+    Constants, Settings
 from pytest_mproc.data import (
     ClientDied,
     GroupTag,
@@ -83,8 +84,9 @@ class RemoteExecutionThread:
             if '@' in destination_text:
                 self._username, port_text = destination_text.split('@', maxsplit=1)
             else:
-                self._username = os.environ.get('SSH_USERNAME')
+                self._username = Settings.ssh_username
                 port_text = destination_text
+            assert self._username
             self._delegation_port = int(port_text) if port_text else find_free_port()
         else:
             self._delegation_port = None
@@ -101,46 +103,17 @@ class RemoteExecutionThread:
         args = (
             server, server_port, self._project_config, self._remote_hosts_config,
             self._remote_sys_executable, self._q, timeout, deploy_timeout,
-            auth_key, hosts_q, port_q, user_output.verbose, Orchestrator.ptmproc_args,
-            self._delegation_port, self._username
+            auth_key, hosts_q, port_q, user_output.verbose,
+            Constants.ptmproc_args,
+            Settings.cache_dir,
+            Settings.tmp_root,
+            self._username, Settings.ssh_password,
+            self._delegation_port,
         )
         self._proc = multiprocessing.Process(target=RemoteExecutionThread._start, args=args)
         self._proc.start()
         return self._proc
 
-    @staticmethod
-    def _determine_cli_args(remote_hosts_config: List[RemoteHostConfig],
-                            ptmproc_args: Dict[str, Any]):
-        args = list(sys.argv[1:])  # copy of
-        # remove pytest_mproc cli args to pass to client (aka additional non-pytest_mproc args)
-        for arg in sys.argv[1:]:
-            typ = ptmproc_args.get(arg)
-            if not typ:
-                continue
-            if arg == "--cores":
-                have_core_count = False
-                for cfg in remote_hosts_config:
-                    if "cores" in cfg.arguments:
-                        have_core_count = True
-                        break
-                if have_core_count:
-                    index = args.index("--cores")
-                    args.remove("--cores")
-                    if index >= 0 and index + 1 < len(args):
-                        args.remove(args[index + 1])
-                    elif index >= len(args):
-                        raise pytest.UsageError(f"--cores specified without a value")
-                continue
-            if typ in (bool,) and arg in args:
-                args.remove(arg)
-            elif arg in args:
-                index = args.index(arg)
-                if index + 1 < len(args):
-                    args.remove(args[index + 1])
-                args.remove(arg)
-        if "--cores" not in args:
-            args += ["--cores", "1"]
-        return args
 
     @classmethod
     def _start(cls, server: str, server_port: int, project_config: ProjectConfig,
@@ -153,9 +126,16 @@ class RemoteExecutionThread:
                port_q: Queue,
                verbose: bool,
                ptmproc_args: Dict[str, Any],
+               cache_dir: Path,
+               tmp_root: Path,
+               username: str,
+               password: Optional[str] = None,
                delegation_port: Optional[int] = None,
-               username: Optional[str] = None
                ):
+        Constants.ptmproc_args = ptmproc_args
+        Settings.set_ssh_credentials(username, password)
+        Settings.set_cache_dir(cache_dir)
+        Settings.set_tmp_root(tmp_root)
         finish_sem = None
         user_output.set_verbose(verbose)
         delegation_proc = None
@@ -164,17 +144,15 @@ class RemoteExecutionThread:
                     Bundle.create(root_dir=Path(tmpdir),
                                   project_config=project_config,
                                   system_executable=remote_sys_executable) as bundle:
-                args = cls._determine_cli_args(remote_hosts_config, ptmproc_args)
                 task = bundle.execute_remote_multi(
-                    *args,
                     auth_key=auth_key,
                     timeout=timeout,
                     deploy_timeout=deploy_timeout,
-                    username=username,
                     server_info=(server, server_port),
                     delegation_port=delegation_port,
                     hosts_q=hosts_q,
-                    port_q=port_q
+                    port_q=port_q,
+                    ptmproc_args=ptmproc_args
                 )
                 procs, delegation_host, delegation_proc = asyncio.get_event_loop().run_until_complete(task)
                 server = delegation_host or server
@@ -226,8 +204,6 @@ class Orchestrator:
     class that acts as Main point of orchestration
     """
 
-    ptmproc_args: Dict[str, Any] = {}
-
     @staticmethod
     def populate_worker_queue(q: Queue, remote_hosts_config, deploy_timeout: float, finish_sem: Semaphore,
                               delegation_q: Queue):
