@@ -1,13 +1,11 @@
 import asyncio
 import logging
-import multiprocessing
 import os
 import platform
 from glob import glob
 from multiprocessing import Queue
 
 import binascii
-import pytest
 import sys
 import tempfile
 import zipfile
@@ -24,7 +22,7 @@ from typing import (
     Union,
 )
 
-from pytest_mproc import user_output, get_auth_key
+from pytest_mproc import user_output, get_auth_key, Settings
 from pytest_mproc.fixtures import Node
 from pytest_mproc.ptmproc_data import RemoteHostConfig, ProjectConfig
 from pytest_mproc.remote.ssh import SSHClient, CommandExecutionFailure, remote_root_context
@@ -38,7 +36,6 @@ lock = RLock()
 
 class Bundle:
 
-    # noinspection SpellCheckingInspection
     CACHE_DIR = Path(os.path.expanduser('~')).absolute() / ".ptmproc"
 
     def __init__(self, root_dir: Path,
@@ -172,9 +169,7 @@ class Bundle:
                         relative_run_dir=relative_run_dir,
                         system_executable=system_executable)
         bundle._sources.extend(project_config.src_paths)
-        os.write(sys.stderr.fileno(), f"\n\n>>> Building bundle.  This may take a little while...\n\n".encode('utf-8'))
-        # site_pkgs = env.set_up_local(project_config=project_config, cache_dir=cls.CACHE_DIR)
-        # bundle._sources.append(site_pkgs)
+        os.write(sys.stderr.fileno(), f"\n\n>>> Building bundle...\n\n".encode('utf-8'))
         always_print(">>> Zipping contents for remote worker...")
         with zipfile.ZipFile(bundle.zip_path, mode='w') as zfile:
             for path in bundle._test_files:
@@ -213,10 +208,10 @@ class Bundle:
             )
             await ssh_client.install_packages(
                 'pytest', 'pytest_mproc',
-                venv = remote_venv,
-                cwd = remote_venv.parent,
-                stdout = stdout,
-                stderr = sys.stderr)
+                venv=remote_venv,
+                cwd=remote_venv.parent,
+                stdout=stdout,
+                stderr=sys.stderr)
 
         return remote_venv
 
@@ -224,6 +219,7 @@ class Bundle:
                      remote_venv: Path, remote_root: Path, timeout: Optional[float] = None) -> None:
         """
         Deploy this bundle to a remote ssh client
+        :param remote_venv: location of Python virtual environment on remote host
         :param ssh_client: where to deploy
         :param timeout: raise TimeoutError if deployment takes too long
         :param remote_root: remote path to where bundle is deployed
@@ -251,7 +247,7 @@ class Bundle:
                 text = ('\n' + (await proc.stdout.read()).decode('utf-8')) if proc.stdout else ""
                 raise CommandExecutionFailure(f"Failed to unzip requirements on remote client {text}", proc.returncode)
             if self._requirements_path:
-                always_print(f"Installing requirements on remote worker {ssh_client.host}...")
+                always_print(f"Installing requirements on remote worker {ssh_client.host} to {remote_venv}...")
                 await ssh_client.install(
                     venv=remote_venv,
                     remote_root=remote_root,
@@ -278,7 +274,8 @@ class Bundle:
         root = Path(__file__).parent.parent.parent
         await ssh_client.mkdir(remote_root / 'site-packages' / 'pytest_mproc' / 'remote', exists_ok=True)
         await ssh_client.push(root / 'pytest_mproc' / '*.py', remote_root / 'site-packages' / 'pytest_mproc' / '.')
-        await ssh_client.push(root / 'pytest_mproc' / 'remote' / '*.py', remote_root / 'site-packages' / 'pytest_mproc' / 'remote' / '.')
+        await ssh_client.push(root / 'pytest_mproc' / 'remote' / '*.py', remote_root / 'site-packages' / 'pytest_mproc'
+                              / 'remote' / '.')
         python = str(remote_venv / 'bin' / 'python3')
         env = {'PYTHONPATH': str(remote_root / 'site-packages'),
                'AUTH_TOKEN_STDIN': '1'}
@@ -297,24 +294,24 @@ class Bundle:
 
     async def execute_remote_multi(
             self,
-            *args,
+            ptmproc_args: Dict[str, Any],
             server_info: Tuple[str, int],
-            username: Optional[str] = None,
-            password: Optional[str] = None,
+            hosts_q: Queue,
+            port_q: Queue,
             deploy_timeout: Optional[float] = FIFTEEN_MINUTES,
             timeout: Optional[float] = None,
             auth_key: Optional[bytes] = None,
             env: Optional[Dict[str, str]] = None,
             delegation_port: Optional[int] = None,
-            hosts_q: Queue,
-            port_q: Queue,
     ) -> Tuple[Dict[str, asyncio.subprocess.Process], str, asyncio.subprocess.Process]:
         """
         deploy and execute to/on multiple host targets concurrently (async)
 
-        :param args: common args to pass to host when executing pytest
-        :param username: optional username for login-based ssh
-        :param password: optional password for login-based ssh
+        :param delegation_port:
+        :param port_q:
+        :param hosts_q:
+        :param server_info:
+        :param ptmproc_args:
         :param auth_key: auth token to use in multiprocessing authentication
         :param deploy_timeout: optional timeout if deployment takes too long
         :param env: dict of environment variables to use on execution
@@ -322,9 +319,9 @@ class Bundle:
         :return: Dict of host, proc where host is host process executed on and proc is the completed process
         """
         tasks: List[asyncio.Task] = []
-        deployments = {}
+        deployments: Dict[str, Tuple[Path, Path]] = {}
         contexts = {}
-        deployment_tasks: Dict[str, asyncio.Task] = {}
+        deployment_tasks: Dict[str, Union[bool, asyncio.Task]] = {}
         delegation_proc: Optional[asyncio.subprocess.Process] = None
         delegation_host: Optional[str] = None
         try:
@@ -335,11 +332,12 @@ class Bundle:
             remote_roots: Dict[str, Path] = {}
             remote_venvs: Dict[str, Path] = {}
             while worker_config is not None:
-                ssh_client = SSHClient(worker_config.remote_host, username=username, password=password)
+                ssh_client = SSHClient(host=worker_config.remote_host, username=Settings.ssh_username,
+                                       password=Settings.ssh_password)
                 if worker_config.remote_host not in contexts:
                     context = remote_root_context(self._project_name, ssh_client, worker_config.remote_root)
                     contexts[worker_config.remote_host] = context
-                    remote_roots[worker_config.remote_host], remote_venv_root = await context.__aenter__()
+                    (remote_roots[worker_config.remote_host], remote_venv_root) = await context.__aenter__()
                     remote_venvs[worker_config.remote_host] = remote_venv_root / 'venv'
                 remote_root = remote_roots[worker_config.remote_host]
                 remote_venv = remote_venvs[worker_config.remote_host]
@@ -354,8 +352,6 @@ class Bundle:
                     port_q.put(server_port)
                     server_host = worker_config.remote_host
                 index += 1
-                cli_args, cli_env = worker_config.argument_env_list()
-                env.update(cli_env)
                 if worker_config.remote_host not in deployment_tasks:
                     deployment_tasks[worker_config.remote_host] = asyncio.create_task(self.deploy(
                         ssh_client=ssh_client,
@@ -364,17 +360,21 @@ class Bundle:
                         timeout=deploy_timeout
                     ))
 
-                async def execute(host: str):
+                async def execute(worker_config: RemoteHostConfig, worker_id: str):
+                    host = worker_config.remote_host
                     # we must wait if deployment is not finished
                     # this setup parallelizes multiple host efficiently
                     if deployment_tasks[host]:
                         await deployment_tasks[host]
                         deployment_tasks[host] = False
+                    args, worker_env = _determine_cli_args(worker_config, ptmproc_args, sys.argv[1:])
+                    env.update(worker_env)
+                    args += ["--as-worker", f"{server_host}:{server_port}"]
                     return await self.execute_remote(
-                        *(list(args) + list(cli_args)) + ["--as-worker", f"{server_host}:{server_port}"],
-                        worker_id=f"Worker-{index}",
+                        *args,
+                        worker_id=worker_id,
                         env=env,
-                        cwd=remote_root / f'Worker-{index}',
+                        cwd=remote_root / worker_id,
                         ssh_client=ssh_client,
                         timeout=timeout,
                         deploy_timeout=deploy_timeout,
@@ -384,7 +384,7 @@ class Bundle:
                         deploy=False
                     )
 
-                task = asyncio.get_event_loop().create_task(execute(worker_config.remote_host))
+                task = asyncio.get_event_loop().create_task(execute(worker_config, worker_id=f"Worker-{index}"))
                 tasks.append(task)
                 if worker_config.remote_host not in deployments:
                     deployments[ssh_client.host] = (remote_root, remote_venv)
@@ -430,6 +430,9 @@ class Bundle:
                              ) -> Tuple[str, asyncio.subprocess.Process]:
         """
         Execute tests on remote host
+        :param deploy: whether to deploy or not (aka assume deployment already taken care of)
+        :param deploy_timeout: timeout on deployments after this many seconds
+        :param cwd: directory to run from
         :param ssh_client: ssh client to use to execute remotely
         :param args: args to pass on cmd line
         :param env: additional environment variables to set (key/value pairs)
@@ -476,7 +479,6 @@ class Bundle:
         try:
             if self.prepare_script:
                 always_print("Preparing environment through prepare script...")
-                # noinspection SpellCheckingInspection
                 proc = await ssh_client.execute_remote_cmd(
                     f"./prepare | tee \"{remote_artifacts_dir}{os.sep}{self._prepare_script.stem}.log\"",
                     prefix_cmd=f"set -o pipefail; ",
@@ -512,7 +514,6 @@ class Bundle:
             if proc and self._finalize_script:
                 always_print("Finalizing worker through finalize script on client %s...", ssh_client.host)
                 with suppress(Exception):
-                    # noinspection SpellCheckingInspection
                     finalize_proc = await ssh_client.execute_remote_cmd(
                         f"./finalize  | tee \"{remote_artifacts_dir}{os.sep}]{self._finalize_script.stem}.log\"",
                         cwd=cwd,
@@ -525,21 +526,23 @@ class Bundle:
                     if finalize_proc.returncode:
                         debug_print(f"!!! WARNING: finalize script failed! ")
             worker_id = worker_id or (f"{ssh_client.host}-{pid}" if pid else ssh_client.host)
-            destination_dir = remote_artifacts_dir.absolute()
+            destination_dir = self._artifacts_path.absolute()
             try:
                 if proc:
                     destination_dir.mkdir(exist_ok=True, parents=True)
                     await ssh_client.mkdir(remote_artifacts_dir, exists_ok=True)
-                    await ssh_client.execute_remote_cmd(cwd=remote_artifacts_dir,
-                                                        command=f"zip -r {str(remote_artifacts_dir / 'artifacts.zip')} *",
-                                                        stdout=stdout,
-                                                        stderr=sys.stderr)
-                    await ssh_client.pull(remote_artifacts_dir / "artifacts.zip", destination_dir / "artifacts.zip",
+                    await ssh_client.execute_remote_cmd(
+                        cwd=remote_artifacts_dir,
+                        command=f"zip -r {str(remote_artifacts_dir / 'artifacts.zip')} *",
+                        stdout=stdout,
+                        stderr=sys.stderr)
+                    await ssh_client.pull(remote_artifacts_dir / "artifacts.zip",
+                                          destination_dir / f"artifacts-{worker_id}.zip",
                                           recursive=False)
+                    with zipfile.ZipFile(destination_dir / f"artifacts-{worker_id}.zip") as zfile:
+                        zfile.extractall(destination_dir)
             except CommandExecutionFailure as e:
-                os.write(sys.stderr.fileno(),
-                         f"\n\n!!! Failed to pull {str(remote_artifacts_dir)} to {destination_dir}:\n"
-                         f"    {e}\n\n".encode('utf-8'))
+                always_print(f"Failed to pull artifacts from worker: {e}")
         return ssh_client.host, proc
 
     async def monitor_remote_execution(self, host: str, *args: str,
@@ -550,18 +553,48 @@ class Bundle:
                                        remote_root: Optional[Path] = None,
                                        env: Optional[Dict[str, str]] = None,
                                        ) -> AsyncGenerator[str, str]:
-        ssh_client = SSHClient(host, username, password)
+        ssh_client = SSHClient(host=host, username=username, password=password)
         env = env or {}
         async with remote_root_context(ssh_client, remote_root) as (remote_root, remote_venv_root):
             if "PYTHONPATH" in env:
                 env["PYTHONPATH"] = str(remote_root / 'site-packages') + ':' + env["PYTHONPATH"]
             else:
                 env["PYTHONPATH"] = str(remote_root / 'site-packages')
-            venv = await self.deploy(ssh_client, remote_root=remote_root, remote_venv=remote_venv_root / 'venv',
-                                     timeout=deploy_timeout)
+            await self.deploy(ssh_client, remote_root=remote_root, remote_venv=remote_venv_root / 'venv',
+                              timeout=deploy_timeout)
+            venv = remote_venv_root / 'venv'
             command = venv / 'bin' / 'python3'
             async for line in ssh_client.monitor_remote_execution(
                     command, '-m', 'pytest', *args, timeout=timeout, env=env,
                     cwd=self.remote_run_dir(remote_root) / self._relative_run_path,
             ):
                 yield line
+
+
+def _determine_cli_args(worker_config: RemoteHostConfig, ptmproc_args: Dict[str, Any], sys_args: List[str]):
+    worker_args, worker_env = worker_config.argument_env_list()
+    # remove pytest_mproc cli args to pass to client (aka additional non-pytest_mproc args)
+    args = list(sys_args)  # make a copy
+    for arg in sys_args:
+        typ = ptmproc_args.get(arg)
+        if not typ:
+            continue
+        if arg in worker_args:
+            try:
+                index = args.index(arg)
+                if typ != bool:
+                    args.remove(args[index + 1])
+                args.remove(args[index])
+            except ValueError:
+                raise ValueError("No argument supplied for --cores option")
+        elif typ in (bool,) and arg in args:
+            args.remove(arg)
+        elif arg in args:
+            index = args.index(arg)
+            if index + 1 < len(args):
+                args.remove(args[index + 1])
+            args.remove(arg)
+    args += worker_args
+    if "--cores" not in args:
+        args += ["--cores", "1"]
+    return args, worker_env
